@@ -12,7 +12,9 @@ import shelve
 import json
 import re
 import socket
+import httplib
 import math
+import mimetypes
 
 from contextlib import closing # for Python2.6 compatibility
 
@@ -25,7 +27,7 @@ from boto.exception import S3ResponseError
 from beefish import decrypt, encrypt_file
 import aaargh
 
-__version__ = "0.3.6"
+__version__ = "0.3.7"
 
 DEFAULT_LOCATION = "us-east-1"
 DEFAULT_DESTINATION = "s3"
@@ -54,43 +56,60 @@ class glacier_shelve(object):
         self.shelve.close()
 
 
-class S3Backend:
-    """
-    Backend to handle S3 upload/download
-    """
-    def __init__(self, conf):
+class BakthatBackend:
+    """Handle Configuration for Backends."""
+    def __init__(self, conf=None, extra_conf=[]):
+        self.custom_conf = None
+        self.conf = {}
         if conf is None:
             try:
-                access_key = config.get("aws", "access_key")
-                secret_key = config.get("aws", "secret_key")
-                bucket = config.get("aws", "s3_bucket")
-                try:
-                    region_name = config.get("aws", "region_name")
-                except ConfigParser.NoOptionError:
-                    region_name = DEFAULT_LOCATION
+                self.conf["access_key"] = config.get("aws", "access_key")
+                self.conf["secret_key"] = config.get("aws", "secret_key")
+                self.conf["region_name"] = config.get("aws", "region_name")
+
+                for key in extra_conf:
+                    try:
+                        self.conf[key] = config.get("aws", key)
+                    except Exception, exc:
+                        log.exception(exc)
+                        log.error("Missing configuration variable")
+                        self.conf[key] = None
+
             except ConfigParser.NoOptionError:
                 log.error("Configuration file not available.")
                 log.info("Use 'bakthat configure' to create one.")
                 return
         else:
-            access_key = conf.get("access_key")
-            secret_key = conf.get("secret_key")
-            bucket = conf.get("bucket")
-            region_name = conf.get("region_name", DEFAULT_LOCATION)
+            self.conf["access_key"] = conf.get("access_key")
+            self.conf["secret_key"] = conf.get("secret_key")
+            self.conf["region_name"] = conf.get("region_name", DEFAULT_LOCATION)
 
-        con = boto.connect_s3(access_key, secret_key)
+            for key in extra_conf:
+                self.conf[key] = conf.get(key)
+
+
+class S3Backend(BakthatBackend):
+    """
+    Backend to handle S3 upload/download
+    """
+    def __init__(self, conf=None):
+        BakthatBackend.__init__(self, conf, extra_conf=["s3_bucket"])
+
+        con = boto.connect_s3(self.conf["access_key"], self.conf["secret_key"])
+
+        region_name = self.conf["region_name"]
         if region_name == DEFAULT_LOCATION:
             region_name = ""
 
         try:
-            self.bucket = con.get_bucket(bucket)
+            self.bucket = con.get_bucket(self.conf["s3_bucket"])
         except S3ResponseError, e:
             if e.code == "NoSuchBucket":
-                self.bucket = con.create_bucket(bucket, location=region_name)
+                self.bucket = con.create_bucket(self.conf["s3_bucket"], location=region_name)
             else:
                 raise e
 
-        self.container = "S3 Bucket: {0}".format(bucket)
+        self.container = "S3 Bucket: {0}".format(self.conf["s3_bucket"])
 
     def download(self, keyname):
         k = Key(self.bucket)
@@ -124,39 +143,21 @@ class S3Backend:
         self.bucket.delete_key(k)
 
 
-
-class GlacierBackend:
+class GlacierBackend(BakthatBackend):
     """
     Backend to handle Glacier upload/download
     """
-    def __init__(self, conf):
-        if conf is None:
-            try:
-                access_key = config.get("aws", "access_key")
-                secret_key = config.get("aws", "secret_key")
-                vault_name = config.get("aws", "glacier_vault")
-                try:
-                    region_name = config.get("aws", "region_name")
-                except ConfigParser.NoOptionError:
-                    region_name = DEFAULT_LOCATION
-            except ConfigParser.NoOptionError:
-                log.error("Configuration file not available.")
-                log.info("Use 'bakthat configure' to create one.")
-                return
-        else:
-            access_key = conf.get("access_key")
-            secret_key = conf.get("secret_key")
-            vault_name = conf.get("vault")
-            region_name = conf.get("region_name", DEFAULT_LOCATION)
+    def __init__(self, conf=None):
+        BakthatBackend.__init__(self, conf, extra_conf=["glacier_vault"])
 
-        con = boto.connect_glacier(aws_access_key_id=access_key,
-                                    aws_secret_access_key=secret_key,
-                                    region_name=region_name)
+        con = boto.connect_glacier(aws_access_key_id=self.conf["access_key"],
+                                    aws_secret_access_key=self.conf["secret_key"],
+                                    region_name=self.conf["region_name"])
 
-        self.conf = conf
-        self.vault = con.create_vault(vault_name)
+        self.vault = con.create_vault(self.conf["glacier_vault"])
         self.backup_key = "bakthat_glacier_inventory"
-        self.container = "Glacier vault: {0}".format(vault_name)
+        self.container = "Glacier vault: {0}".format(self.conf["glacier_vault"])
+
 
     def backup_inventory(self):
         """
@@ -259,7 +260,7 @@ class GlacierBackend:
                 try:
                     job = self.vault.get_job(job_id)
                 except UnexpectedHTTPResponseError: # Return a 404 if the job is no more available
-                    del job[keyname]
+                    del jobs[keyname]
 
             if not job:
                 # Job initialization
@@ -280,7 +281,7 @@ class GlacierBackend:
             chunk_size = 4 * 1024 * 1024
             num_chunks = int(math.ceil(job.archive_size / float(chunk_size)))
             job._download_to_fileob(encrypted_out, num_chunks, chunk_size,
-                                     True, (socket.error,))
+                                     True, (socket.error, httplib.IncompleteRead))
 
             encrypted_out.seek(0)
             return encrypted_out
@@ -371,7 +372,8 @@ def backup(filename, destination=None, description=None, prompt="yes", **kwargs)
 
     log.info("Backing up " + filename)
     arcname = filename.strip('/').split('/')[-1]
-    stored_filename = arcname +  datetime.now().strftime("%Y%m%d%H%M%S") + ".tgz"
+    date_component = datetime.utcnow().strftime("%Y%m%d%H%M%S")
+    stored_filename = arcname +  date_component + ".tgz"
     
     password = kwargs.get("password")
     if password is None and prompt.lower() != "no":
@@ -382,23 +384,44 @@ def backup(filename, destination=None, description=None, prompt="yes", **kwargs)
                 log.error("Password confirmation doesn't match")
                 return
 
-    log.info("Compressing...")
-    with tempfile.NamedTemporaryFile(delete=False) as out:
-        with closing(tarfile.open(fileobj=out, mode="w:gz")) as tar:
-            tar.add(filename, arcname=arcname)
-        outname = out.name
+    if mimetypes.guess_type(arcname) == ('application/x-tar', 'gzip'):
+        log.info("File already compressed")
+        outname = filename
 
+        new_arcname = re.sub(r'(\.t(ar\.)?gz)', '', arcname)
+        stored_filename = new_arcname + date_component + ".tgz"
+
+        bakthat_compression = False
+    else:
+        log.info("Compressing...")
+        with tempfile.NamedTemporaryFile(delete=False) as out:
+            with closing(tarfile.open(fileobj=out, mode="w:gz")) as tar:
+                tar.add(filename, arcname=arcname)
+            outname = out.name
+
+        bakthat_compression = True
+
+    bakthat_encryption = False
     if password:
         log.info("Encrypting...")
         encrypted_out = tempfile.NamedTemporaryFile(delete=False)
         encrypt_file(outname, encrypted_out.name, password)
         stored_filename += ".enc"
-        os.remove(outname)  # remove non-encrypted tmp file
+
+        # We only remove the file if the archive is created by bakthat
+        if bakthat_compression:
+            os.remove(outname)  # remove non-encrypted tmp file
+
         outname = encrypted_out.name
 
+        bakthat_encryption = True
+
     log.info("Uploading...")
     storage_backend.upload(stored_filename, outname)
-    os.remove(outname)
+
+    # We only remove the file if the archive is created by bakthat
+    if bakthat_encryption:
+        os.remove(outname)
 
     return True
 
